@@ -16,6 +16,9 @@ from time import time, sleep
 from codecs import open
 
 
+from cli_helpers.tabular_output import TabularOutputFormatter
+from cli_helpers.tabular_output.preprocessors import (align_decimals,
+                                                      format_numbers)
 import click
 try:
     import setproctitle
@@ -34,9 +37,7 @@ from prompt_toolkit.history import FileHistory
 from pygments.lexers.sql import PostgresLexer
 from pygments.token import Token
 
-from .packages.tabulate import tabulate
-from .packages.expanded import expanded_table
-from pgspecial.main import (PGSpecial, NO_QUERY, content_exceeds_width)
+from pgspecial.main import (PGSpecial, NO_QUERY)
 import pgspecial as special
 from .pgcompleter import PGCompleter
 from .pgtoolbar import create_toolbar_tokens_func
@@ -58,7 +59,7 @@ except ImportError:
     from urllib.parse import urlparse, unquote, parse_qs
 
 from getpass import getuser
-from psycopg2 import OperationalError
+from psycopg2 import OperationalError, InterfaceError
 
 from collections import namedtuple
 
@@ -84,11 +85,6 @@ OutputSettings.__new__.__defaults__ = (
     None, None, None, '<null>', False, None, lambda x: x
 )
 
-# no-op logging handler
-class NullHandler(logging.Handler):
-    def emit(self, record):
-        pass
-
 
 class PGCli(object):
 
@@ -100,16 +96,21 @@ class PGCli(object):
         os_environ_pager = os.environ.get('PAGER')
 
         if configured_pager:
-            self.logger.info('Default pager found in config file: ' + '\'' + configured_pager + '\'')
+            self.logger.info(
+                'Default pager found in config file: "{}"'.format(configured_pager))
             os.environ['PAGER'] = configured_pager
         elif os_environ_pager:
-            self.logger.info('Default pager found in PAGER environment variable: ' + '\'' + os_environ_pager + '\'')
+            self.logger.info('Default pager found in PAGER environment variable: "{}"'.format(
+                os_environ_pager))
             os.environ['PAGER'] = os_environ_pager
         else:
-            self.logger.info('No default pager found in environment. Using os default pager')
-        # Always set default set of less recommended options, they are ignored if pager is
-        # different than less or is already parameterized with their own arguments
-        os.environ['LESS'] = '-SRXF'
+            self.logger.info(
+                'No default pager found in environment. Using os default pager')
+
+        # Set default set of less recommended options, if they are not already set.
+        # They are ignored if pager is different than less.
+        if not os.environ.get('LESS'):
+            os.environ['LESS'] = '-SRXF'
 
     def __init__(self, force_passwd_prompt=False, never_passwd_prompt=False,
                  pgexecute=None, pgclirc_file=None, row_limit=None,
@@ -255,7 +256,7 @@ class PGCli(object):
         # Disable logging if value is NONE by switching to a no-op handler.
         # Set log level to a high value so it doesn't even waste cycles getting called.
         if log_level.upper() == 'NONE':
-            handler = NullHandler()
+            handler = logging.NullHandler()
         else:
             handler = logging.FileHandler(os.path.expanduser(log_file))
 
@@ -346,7 +347,7 @@ class PGCli(object):
             try:
                 pgexecute = PGExecute(database, user, passwd, host, port, dsn,
                                       **kwargs)
-            except OperationalError as e:
+            except (OperationalError, InterfaceError) as e:
                 if ('no password supplied' in utf8tounicode(e.args[0]) and
                         auto_passwd_prompt):
                     passwd = click.prompt('Password', hide_input=True,
@@ -758,7 +759,6 @@ class PGCli(object):
         return self.query_history[-1][0] if self.query_history else None
 
 
-
 @click.command()
 # Default host is '' so psycopg2 can default to either localhost or unix socket
 @click.option('-h', '--host', default='', envvar='PGHOST',
@@ -787,11 +787,13 @@ class PGCli(object):
         default=False,
         help='Skip intro on startup and goodbye on exit.')
 @click.option('--prompt', help='Prompt format (Default: "\\u@\\h:\\d> ").')
+@click.option('-l', '--list', 'list_databases', is_flag=True, help='list '
+              'available databases, then exit.')
 @click.argument('database', default=lambda: None, envvar='PGDATABASE', nargs=1)
 @click.argument('username', default=lambda: None, envvar='PGUSER', nargs=1)
 def cli(database, username_opt, host, port, prompt_passwd, never_prompt,
-    single_connection, dbname, username, version, pgclirc, dsn, row_limit,
-    less_chatty, prompt):
+        single_connection, dbname, username, version, pgclirc, dsn, row_limit,
+        less_chatty, prompt, list_databases):
 
     if version:
         print('Version:', __version__)
@@ -823,7 +825,7 @@ def cli(database, username_opt, host, port, prompt_passwd, never_prompt,
 
     if dsn is not '':
         try:
-            cfg = load_config(config_full_path)
+            cfg = load_config(pgclirc, config_full_path)
             dsn_config = cfg['alias_dsn'][dsn]
         except:
             click.secho('Invalid DSNs found in the config file. '\
@@ -840,6 +842,19 @@ def cli(database, username_opt, host, port, prompt_passwd, never_prompt,
     else:
         pgcli.connect(database, host, user, port)
 
+    if list_databases:
+        cur, headers, status = pgcli.pgexecute.full_databases()
+
+        title = 'List of databases'
+        settings = OutputSettings(
+            table_format='ascii',
+            missingval='<null>'
+        )
+        formatted = format_output(title, cur, headers, status, settings)
+        click.echo_via_pager('\n'.join(formatted))
+
+        sys.exit(0)
+
     pgcli.logger.debug('Launch Params: \n'
             '\tdatabase: %r'
             '\tuser: %r'
@@ -862,41 +877,12 @@ def obfuscate_process_password():
     setproctitle.setproctitle(process_title)
 
 
-def format_output(title, cur, headers, status, settings):
-    output = []
-    missingval = settings.missingval
-    table_format = settings.table_format
-    dcmlfmt = settings.dcmlfmt
-    floatfmt = settings.floatfmt
-    expanded = settings.expanded
-    max_width = settings.max_width
-    case_function = settings.case_function
-    if title:  # Only print the title if it's not None.
-        output.append(title)
-    if cur:
-        headers = [case_function(utf8tounicode(x)) for x in headers]
-        if expanded and headers:
-            output.append(expanded_table(cur, headers, missingval))
-        else:
-            tabulated, rows = tabulate(cur, headers, tablefmt=table_format,
-                missingval=missingval, dcmlfmt=dcmlfmt, floatfmt=floatfmt)
-            if (max_width and rows and
-                    content_exceeds_width(rows[0], max_width) and
-                    headers):
-                output.append(expanded_table(rows, headers, missingval))
-            else:
-                output.append(tabulated)
-    if status:  # Only print the status if it's not None.
-        output.append(status)
-    return output
-
-
 def has_meta_cmd(query):
     """Determines if the completion needs a refresh by checking if the sql
-    statement is an alter, create, or drop"""
+    statement is an alter, create, drop, commit or rollback."""
     try:
         first_token = query.split()[0]
-        if first_token.lower() in ('alter', 'create', 'drop'):
+        if first_token.lower() in ('alter', 'create', 'drop', 'commit', 'rollback'):
             return True
     except Exception:
         return False
@@ -949,5 +935,49 @@ def exception_formatter(e):
     return click.style(utf8tounicode(str(e)), fg='red')
 
 
+def format_output(title, cur, headers, status, settings):
+    output = []
+    expanded = (settings.expanded or settings.table_format == 'vertical')
+    table_format = ('vertical' if settings.expanded else
+                    settings.table_format)
+    max_width = settings.max_width
+    case_function = settings.case_function
+    formatter = TabularOutputFormatter(format_name=table_format)
+
+    output_kwargs = {
+        'sep_title': 'RECORD {n}',
+        'sep_character': '-',
+        'sep_length': (1, 25),
+        'missing_value': settings.missingval,
+        'integer_format': settings.dcmlfmt,
+        'float_format': settings.floatfmt,
+        'preprocessors': (format_numbers, ),
+        'disable_numparse': True,
+        'preserve_whitespace': True
+    }
+    if not settings.floatfmt:
+        output_kwargs['preprocessors'] = (align_decimals, )
+
+    if title:  # Only print the title if it's not None.
+        output.append(title)
+
+    if cur:
+        headers = [case_function(utf8tounicode(x)) for x in headers]
+        rows = list(cur)
+        formatted = formatter.format_output(rows, headers, **output_kwargs)
+        first_line = formatted[:formatted.find('\n')]
+
+        if (not expanded and max_width and len(first_line) > max_width and headers):
+            formatted = formatter.format_output(
+                rows, headers, format_name='vertical', **output_kwargs)
+
+        output.append(formatted)
+
+    if status:  # Only print the status if it's not None.
+        output.append(status)
+
+    return output
+
+
 if __name__ == "__main__":
     cli()
