@@ -21,7 +21,8 @@ else:
 
 Special = namedtuple('Special', [])
 Database = namedtuple('Database', [])
-Schema = namedtuple('Schema', [])
+Schema = namedtuple('Schema', ['quoted'])
+Schema.__new__.__defaults__ = (False,)
 # FromClauseItem is a table/view/function used in the FROM clause
 # `table_refs` contains the list of tables/... already in the statement,
 # used to ensure that the alias we suggest is unique
@@ -42,9 +43,9 @@ FromClauseItem.__new__.__defaults__ = (None, tuple(), tuple())
 
 Column = namedtuple(
     'Column',
-    ['table_refs', 'require_last_table', 'local_tables', 'qualifiable']
+    ['table_refs', 'require_last_table', 'local_tables', 'qualifiable', 'context']
 )
-Column.__new__.__defaults__ = (None, None, tuple(), False)
+Column.__new__.__defaults__ = (None, None, tuple(), False, None)
 
 Keyword = namedtuple('Keyword', ['last_token'])
 Keyword.__new__.__defaults__ = (None,)
@@ -169,6 +170,7 @@ def _strip_named_query(txt):
         txt = named_query_regex.sub('', txt)
     return txt
 
+
 function_body_pattern = re.compile('(\\$.*?\\$)([\s\S]*?)\\1', re.M)
 
 
@@ -235,6 +237,7 @@ SPECIALS_SUGGESTION = {
     'sf': Function,
 }
 
+
 def suggest_special(text):
     text = text.lstrip()
     cmd, _, arg = parse_special_command(text)
@@ -374,7 +377,9 @@ def suggest_based_on_last_token(token, stmt):
                 return (Keyword(),)
         prev_prev_tok = prev_tok and p.token_prev(p.token_index(prev_tok))[1]
         if prev_prev_tok and prev_prev_tok.normalized == 'INTO':
-            return (Column(table_refs=stmt.get_tables('insert')),)
+            return (
+                Column(table_refs=stmt.get_tables('insert'), context='insert'),
+            )
         # We're probably in a function argument list
         return (Column(table_refs=extract_tables(stmt.full_text),
                        local_tables=stmt.local_tables, qualifiable=True),)
@@ -473,8 +478,10 @@ def suggest_based_on_last_token(token, stmt):
         # "CREATE DATABASE <newdb> WITH TEMPLATE <db>"
         return (Database(),)
     elif token_v == 'schema':
-        # DROP SCHEMA schema_name
-        return (Schema(),)
+        # DROP SCHEMA schema_name, SET SCHEMA schema name
+        prev_keyword = stmt.reduce_to_prev_keyword(n_skip=2)
+        quoted = prev_keyword and prev_keyword.value.lower() == 'set'
+        return (Schema(quoted),)
     elif token_v.endswith(',') or token_v in ('=', 'and', 'or'):
         prev_keyword = stmt.reduce_to_prev_keyword()
         if prev_keyword:
