@@ -146,6 +146,16 @@ class PGExecute(object):
         FROM pg_catalog.pg_database d
         ORDER BY 1'''
 
+    full_databases_query = '''
+        SELECT d.datname as "Name",
+            pg_catalog.pg_get_userbyid(d.datdba) as "Owner",
+            pg_catalog.pg_encoding_to_char(d.encoding) as "Encoding",
+            d.datcollate as "Collate",
+            d.datctype as "Ctype",
+            pg_catalog.array_to_string(d.datacl, E'\n') AS "Access privileges"
+        FROM pg_catalog.pg_database d
+        ORDER BY 1'''
+
     def __init__(self, database, user, password, host, port, dsn, **kwargs):
         self.dbname = database
         self.user = user
@@ -415,12 +425,17 @@ class PGExecute(object):
                 SELECT  nsp.nspname schema_name,
                         cls.relname table_name,
                         att.attname column_name,
-                        att.atttypid::regtype::text type_name
+                        att.atttypid::regtype::text type_name,
+                        att.atthasdef AS has_default,
+                        def.adsrc as default
                 FROM    pg_catalog.pg_attribute att
                         INNER JOIN pg_catalog.pg_class cls
                             ON att.attrelid = cls.oid
                         INNER JOIN pg_catalog.pg_namespace nsp
                             ON cls.relnamespace = nsp.oid
+                        LEFT OUTER JOIN pg_attrdef def
+                            ON def.adrelid = att.attrelid
+                            AND def.adnum = att.attnum
                 WHERE   cls.relkind = ANY(%s)
                         AND NOT att.attisdropped
                         AND att.attnum  > 0
@@ -430,7 +445,9 @@ class PGExecute(object):
                 SELECT  nsp.nspname schema_name,
                         cls.relname table_name,
                         att.attname column_name,
-                        typ.typname type_name
+                        typ.typname type_name,
+                        NULL AS has_default,
+                        NULL AS default
                 FROM    pg_catalog.pg_attribute att
                         INNER JOIN pg_catalog.pg_class cls
                             ON att.attrelid = cls.oid
@@ -464,6 +481,14 @@ class PGExecute(object):
             cur.execute(self.databases_query)
             return [x[0] for x in cur.fetchall()]
 
+    def full_databases(self):
+        with self.conn.cursor() as cur:
+            _logger.debug('Databases Query. sql: %r',
+                          self.full_databases_query)
+            cur.execute(self.full_databases_query)
+            headers = [x[0] for x in cur.description]
+            return cur.fetchall(), headers, cur.statusmessage
+
     def foreignkeys(self):
         """Yields ForeignKey named tuples"""
 
